@@ -255,7 +255,7 @@ static int enc_speech_codec(struct msgb *msg,
 		header |= (1 << 4);
 
 	if (type_extended) {
-		header |= 0x0f;
+		header |= GSM0808_SCT_EXT;
 		msgb_put_u8(msg, header);
 		msgb_put_u8(msg, sc->type);
 	} else {
@@ -347,7 +347,7 @@ int gsm0808_dec_speech_codec(struct gsm0808_speech_codec *sc,
 
 	/* An extended codec type needs at least two fields,
 	 * bail if the input data length is not sufficient. */
-	if ((header & 0x0F) == 0x0F && len < 2)
+	if ((header & 0x0F) == GSM0808_SCT_EXT && len < 2)
 		return -EINVAL;
 
 	elem++;
@@ -362,7 +362,7 @@ int gsm0808_dec_speech_codec(struct gsm0808_speech_codec *sc,
 	if (header & (1 << 4))
 		sc->tf = true;
 
-	if ((header & 0x0F) != 0x0F) {
+	if ((header & 0x0F) != GSM0808_SCT_EXT) {
 		sc->type = (header & 0x0F);
 	} else {
 		sc->type = *elem;
@@ -499,12 +499,6 @@ uint8_t gsm0808_enc_channel_type(struct msgb *msg,
 
 	OSMO_ASSERT(ct->perm_spch_len <= CHANNEL_TYPE_ELEMENT_MAXLEN - 2);
 
-	/* FIXME: Implement encoding support for Data
-	 * and Speech + CTM Text Telephony */
-	if ((ct->ch_indctr & 0x0f) != GSM0808_CHAN_SPEECH
-	    && (ct->ch_indctr & 0x0f) != GSM0808_CHAN_SIGN)
-		OSMO_ASSERT(false);
-
 	msgb_put_u8(msg, GSM0808_IE_CHANNEL_TYPE);
 	tlv_len = msgb_put(msg, 1);
 	old_tail = msg->tail;
@@ -512,12 +506,45 @@ uint8_t gsm0808_enc_channel_type(struct msgb *msg,
 	msgb_put_u8(msg, ct->ch_indctr & 0x0f);
 	msgb_put_u8(msg, ct->ch_rate_type);
 
-	for (i = 0; i < ct->perm_spch_len; i++) {
-		byte = ct->perm_spch[i];
+	switch (ct->ch_indctr) {
+	case GSM0808_CHAN_DATA:
+		byte = ct->data_rate;
 
-		if (i < ct->perm_spch_len - 1)
-			byte |= 0x80;
+		if (!ct->data_transparent)
+			byte |= 0x40; /* Set T/NT */
+
+		if (ct->data_rate_allowed_is_set) {
+			OSMO_ASSERT(!ct->data_transparent);
+			byte |= 0x80; /* Set ext */
+			msgb_put_u8(msg, byte);
+
+			byte = ct->data_rate_allowed;
+			if (ct->data_asym_pref_is_set) {
+				byte |= 0x80; /* Set ext */
+				msgb_put_u8(msg, byte);
+
+				/* Set asymmetry indication, rest is spare */
+				byte = ct->data_asym_pref << 5;
+			}
+		}
 		msgb_put_u8(msg, byte);
+		break;
+	case GSM0808_CHAN_SPEECH:
+	case GSM0808_CHAN_SPEECH_CTM_TEXT_TELEPHONY:
+		for (i = 0; i < ct->perm_spch_len; i++) {
+			byte = ct->perm_spch[i];
+
+			if (i < ct->perm_spch_len - 1)
+				byte |= 0x80;
+			msgb_put_u8(msg, byte);
+		}
+		break;
+	case GSM0808_CHAN_SIGN:
+		/* Octet 5 is spare */
+		msgb_put_u8(msg, 0);
+		break;
+	default:
+		OSMO_ASSERT(false);
 	}
 
 	*tlv_len = (uint8_t) (msg->tail - old_tail);
@@ -546,17 +573,60 @@ int gsm0808_dec_channel_type(struct gsm0808_channel_type *ct,
 
 	ct->ch_indctr = (*elem) & 0x0f;
 	elem++;
-	ct->ch_rate_type = (*elem) & 0x0f;
+	ct->ch_rate_type = *elem;
 	elem++;
 
-	for (i = 0; i < ARRAY_SIZE(ct->perm_spch); i++) {
+	switch (ct->ch_indctr) {
+	case GSM0808_CHAN_DATA:
 		byte = *elem;
 		elem++;
-		ct->perm_spch[i] = byte & 0x7f;
-		if ((byte & 0x80) == 0x00)
-			break;
+		ct->data_transparent = !(byte & 0x40); /* T/NT */
+		ct->data_rate = byte & 0x3f;
+
+		/* Optional extension for non-transparent service */
+		if (byte & 0x80) {
+			if (ct->data_transparent)
+				return -EINVAL;
+			if (elem - old_elem >= len)
+				return -EOVERFLOW;
+			byte = *elem;
+			elem++;
+
+			ct->data_rate_allowed_is_set = true;
+			ct->data_rate_allowed = byte & 0x7f;
+
+			/* Optional extension */
+			if (byte & 0x80) {
+				if (elem - old_elem >= len)
+					return -EOVERFLOW;
+				byte = *elem;
+				elem++;
+
+				ct->data_asym_pref_is_set = true;
+				ct->data_asym_pref = byte & 0x60 >> 5;
+			}
+		}
+		break;
+	case GSM0808_CHAN_SPEECH:
+	case GSM0808_CHAN_SPEECH_CTM_TEXT_TELEPHONY:
+		for (i = 0; i < ARRAY_SIZE(ct->perm_spch); i++) {
+			if (elem - old_elem >= len)
+				return -EOVERFLOW;
+
+			byte = *elem;
+			elem++;
+			ct->perm_spch[i] = byte & 0x7f;
+			if ((byte & 0x80) == 0x00)
+				break;
+		}
+		ct->perm_spch_len = i + 1;
+		break;
+	case GSM0808_CHAN_SIGN:
+		/* Octet 5 is spare */
+		break;
+	default:
+		return -ENOTSUP;
 	}
-	ct->perm_spch_len = i + 1;
 
 	return (int)(elem - old_elem);
 }
@@ -2005,25 +2075,12 @@ const struct value_string gsm0808_cell_id_discr_names[] = {
 	{ 0, NULL }
 };
 
-#define APPEND_THING(func, args...) do { \
-		int remain = buflen - (pos - buf); \
-		int l = func(pos, remain, ##args); \
-		if (l < 0 || l > remain) \
-			pos = buf + buflen; \
-		else \
-			pos += l; \
-		if (l > 0) \
-			total_len += l; \
-	} while(0)
-#define APPEND_STR(fmt, args...) APPEND_THING(snprintf, fmt, ##args)
-#define APPEND_CELL_ID_U(DISCR, U) APPEND_THING(gsm0808_cell_id_u_name, DISCR, U)
-
 char *gsm0808_cell_id_name_buf(char *buf, size_t buflen, const struct gsm0808_cell_id *cid)
 {
-	char *pos = buf;
-	int total_len = 0;
-	APPEND_STR("%s:", gsm0808_cell_id_discr_name(cid->id_discr));
-	APPEND_CELL_ID_U(cid->id_discr, &cid->id);
+	struct osmo_strbuf sb = { .buf = buf, .len = buflen };
+
+	OSMO_STRBUF_PRINTF(sb, "%s:", gsm0808_cell_id_discr_name(cid->id_discr));
+	OSMO_STRBUF_APPEND(sb, gsm0808_cell_id_u_name, cid->id_discr, &cid->id);
 	return buf;
 }
 
@@ -2070,30 +2127,31 @@ char *gsm0808_cell_id_name_c(const void *ctx, const struct gsm0808_cell_id *cid)
  */
 int gsm0808_cell_id_list_name_buf(char *buf, size_t buflen, const struct gsm0808_cell_id_list2 *cil)
 {
-	char *pos = buf;
-	int total_len = 0;
-	int i;
+	struct osmo_strbuf sb = { .buf = buf, .len = buflen };
 
-	APPEND_STR("%s[%u]", gsm0808_cell_id_discr_name(cil->id_discr), cil->id_list_len);
+	OSMO_STRBUF_PRINTF(sb, "%s[%u]",
+			   gsm0808_cell_id_discr_name(cil->id_discr),
+			   cil->id_list_len);
 
 	switch (cil->id_discr) {
 	case CELL_IDENT_BSS:
 	case CELL_IDENT_NO_CELL:
-		return total_len;
+		return sb.chars_needed;
 	default:
 		break;
 	}
 
-	APPEND_STR(":{");
+	OSMO_STRBUF_PRINTF(sb, ":{");
 
-	for (i = 0; i < cil->id_list_len; i++) {
+	for (unsigned int i = 0; i < cil->id_list_len; i++) {
 		if (i)
-			APPEND_STR(", ");
-		APPEND_CELL_ID_U(cil->id_discr, &cil->id_list[i]);
+			OSMO_STRBUF_PRINTF(sb, ", ");
+		OSMO_STRBUF_APPEND(sb, gsm0808_cell_id_u_name,
+				   cil->id_discr, &cil->id_list[i]);
 	}
 
-	APPEND_STR("}");
-	return total_len;
+	OSMO_STRBUF_PRINTF(sb, "}");
+	return sb.chars_needed;
 }
 
 /*! Return a human-readable representation of \a cil in a static buffer.
@@ -2115,9 +2173,6 @@ char *gsm0808_cell_id_list_name_c(const void *ctx, const struct gsm0808_cell_id_
 	return buf;
 }
 
-#undef APPEND_STR
-#undef APPEND_CELL_ID_U
-
 char *gsm0808_channel_type_name_buf(char *buf, size_t buf_len, const struct gsm0808_channel_type *ct)
 {
 	snprintf(buf, buf_len, "ch_indctr=0x%x ch_rate_type=0x%x perm_spch=%s",
@@ -2140,4 +2195,367 @@ char *gsm0808_channel_type_name_c(const void *ctx, const struct gsm0808_channel_
 	return gsm0808_channel_type_name_buf(buf, 128, ct);
 }
 
+/*! Encode Group Call Reference IE (3GPP TS 48.008 3.2.2.55).
+ *  \param[out] msg Message Buffer to which IE is to be appended
+ *  \param[in] gc Group Call Reference to be encoded
+ *  \returns number of bytes appended to \a msg */
+uint8_t gsm0808_enc_group_callref(struct msgb *msg, const struct gsm0808_group_callref *gc)
+{
+	uint8_t *old_tail;
+	uint8_t *tlv_len;
+
+	OSMO_ASSERT(msg);
+	OSMO_ASSERT(gc);
+
+	msgb_put_u8(msg, GSM0808_IE_GROUP_CALL_REFERENCE);
+	tlv_len = msgb_put(msg, 1);
+	old_tail = msg->tail;
+
+	memcpy(msgb_put(msg, sizeof(*gc)), gc, sizeof(*gc));
+
+	*tlv_len = (uint8_t) (msg->tail - old_tail);
+	return *tlv_len + 2;
+}
+
+/*! Decode Group Call Reference IE (3GPP TS 48.008 3.2.2.55).
+ *  \param[out] gc Group Call Reference structure to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_group_callref(struct gsm0808_group_callref *gc, const uint8_t *elem, uint8_t len)
+{
+	OSMO_ASSERT(gc);
+	OSMO_ASSERT(elem);
+
+	if (len != sizeof(*gc))
+		return -EINVAL;
+
+	memcpy(gc, elem, sizeof(*gc));
+
+	return len;
+}
+
+/*! Encode Priority IE (3GPP TS 48.008 3.2.2.18).
+ *  \param[out] msg Message Buffer to which IE is to be appended
+ *  \param[in] pri Priority to be encoded
+ *  \returns number of bytes appended to \a msg */
+uint8_t gsm0808_enc_priority(struct msgb *msg, const struct gsm0808_priority *pri)
+{
+	uint8_t *old_tail;
+	uint8_t *tlv_len;
+
+	OSMO_ASSERT(msg);
+	OSMO_ASSERT(pri);
+
+	msgb_put_u8(msg, GSM0808_IE_PRIORITY);
+	tlv_len = msgb_put(msg, 1);
+	old_tail = msg->tail;
+
+	memcpy(msgb_put(msg, sizeof(*pri)), pri, sizeof(*pri));
+
+	*tlv_len = (uint8_t) (msg->tail - old_tail);
+	return *tlv_len + 2;
+}
+
+/*! Decode Priority IE (3GPP TS 48.008 3.2.2.18).
+ *  \param[out] pri Priority structure to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_priority(struct gsm0808_priority *pri, const uint8_t *elem, uint8_t len)
+{
+	OSMO_ASSERT(pri);
+	OSMO_ASSERT(elem);
+
+	if (len != sizeof(*pri))
+		return -EINVAL;
+
+	memcpy(pri, elem, sizeof(*pri));
+
+	return len;
+}
+
+/*! Encode VGCS Feature Flags IE (3GPP TS 48.008 3.2.2.88).
+ *  \param[out] msg Message Buffer to which IE is to be appended
+ *  \param[in] ff VGCS Feature Flags to be encoded
+ *  \returns number of bytes appended to \a msg */
+uint8_t gsm0808_enc_vgcs_feature_flags(struct msgb *msg, const struct gsm0808_vgcs_feature_flags *ff)
+{
+	uint8_t *old_tail;
+	uint8_t *tlv_len;
+
+	OSMO_ASSERT(msg);
+	OSMO_ASSERT(ff);
+
+	msgb_put_u8(msg, GSM0808_IE_VGCS_FEATURE_FLAGS);
+	tlv_len = msgb_put(msg, 1);
+	old_tail = msg->tail;
+
+	memcpy(msgb_put(msg, sizeof(*ff)), ff, sizeof(*ff));
+
+	*tlv_len = (uint8_t) (msg->tail - old_tail);
+	return *tlv_len + 2;
+}
+
+/*! Decode VGCS Feature Flags IE (3GPP TS 48.008 3.2.2.88).
+ *  \param[out] ff VGCS Feature Flags structure to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_vgcs_feature_flags(struct gsm0808_vgcs_feature_flags *ff, const uint8_t *elem, uint8_t len)
+{
+	OSMO_ASSERT(ff);
+	OSMO_ASSERT(elem);
+
+	if (len != sizeof(*ff))
+		return -EINVAL;
+
+	memcpy(ff, elem, sizeof(*ff));
+
+	return len;
+}
+
+/*! Encode Data Identity IE (3GPP TS 48.008 3.2.2.99).
+ *  \param[out] msg Message Buffer to which IE is to be appended
+ *  \param[in] di Data Identity to be encoded
+ *  \returns number of bytes appended to \a msg */
+uint8_t gsm0808_enc_data_identity(struct msgb *msg, const struct gsm0808_data_identity *ti)
+{
+	uint8_t *old_tail;
+	uint8_t *tlv_len;
+
+	OSMO_ASSERT(msg);
+	OSMO_ASSERT(ti);
+
+	msgb_put_u8(msg, GSM0808_IE_DATA_IDENTITY);
+	tlv_len = msgb_put(msg, 1);
+	old_tail = msg->tail;
+
+	memcpy(msgb_put(msg, sizeof(*ti)), ti, sizeof(*ti));
+
+	*tlv_len = (uint8_t) (msg->tail - old_tail);
+	return *tlv_len + 2;
+}
+
+/*! Decode Data Identity IE (3GPP TS 48.008 3.2.2.99).
+ *  \param[out] di Data Identity structure to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_data_identity(struct gsm0808_data_identity *ti, const uint8_t *elem, uint8_t len)
+{
+	OSMO_ASSERT(ti);
+	OSMO_ASSERT(elem);
+
+	if (len < sizeof(*ti))
+		return -EINVAL;
+
+	memcpy(ti, elem, sizeof(*ti));
+
+	return len;
+}
+
+/*! Encode MSISDN IE (3GPP TS 48.008 3.2.2.101).
+ *  \param[out] msg Message Buffer to which IE is to be appended
+ *  \param[in] msisdn MSISDN to be encoded
+ *  \returns number of bytes appended to \a msg */
+uint8_t gsm0808_enc_msisdn(struct msgb *msg, const char *msisdn)
+{
+	uint8_t *tlv_len;
+	uint8_t bcd_lv[11];
+	int rc;
+
+	OSMO_ASSERT(msg);
+	OSMO_ASSERT(msisdn);
+
+	rc = gsm48_encode_bcd_number(bcd_lv, sizeof(bcd_lv), 0, msisdn);
+	if (rc < 1)
+		return 0;
+
+	msgb_put_u8(msg, GSM0808_IE_MSISDN);
+	tlv_len = msgb_put(msg, rc);
+
+	memcpy(tlv_len, bcd_lv, rc);
+
+	*tlv_len = rc - 1;
+	return *tlv_len + 2;
+}
+
+/*! Decode MSISDN IE (3GPP TS 48.008 3.2.2.101).
+ *  \param[out] msisdn MSISDN structure to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_msisdn(char *msisdn, const char *elem, uint8_t len)
+{
+	OSMO_ASSERT(msisdn);
+	OSMO_ASSERT(elem);
+
+	if (len < 1)
+		return -EINVAL;
+
+	gsm48_decode_bcd_number(msisdn, MSISDN_MAXLEN + 1, (uint8_t *)(elem - 1), 0);
+
+	return len;
+}
+
+/*! Encode Talker Identity IE (3GPP TS 48.008 3.2.2.91).
+ *  \param[out] msg Message Buffer to which IE is to be appended
+ *  \param[in] ti Talker Identity to be encoded
+ *  \returns number of bytes appended to \a msg */
+uint8_t gsm0808_enc_talker_identity(struct msgb *msg, const struct gsm0808_talker_identity *ti)
+{
+	uint8_t *old_tail;
+	uint8_t *tlv_len;
+	uint8_t *ptr;
+	unsigned int bytes;
+
+	OSMO_ASSERT(msg);
+	OSMO_ASSERT(ti);
+
+	msgb_put_u8(msg, GSM0808_IE_TALKER_IDENTITY);
+	tlv_len = msgb_put(msg, 1);
+	old_tail = msg->tail;
+
+	bytes = (ti->id_bits + 7) >> 3;
+	ptr = msgb_put(msg, 1 + bytes);
+	ptr[0] = -ti->id_bits & 0x7;
+	memcpy(ptr + 1, ti->talker_id, bytes);
+
+	*tlv_len = (uint8_t) (msg->tail - old_tail);
+	return *tlv_len + 2;
+}
+
+/*! Decode Talker Identity IE (3GPP TS 48.008 3.2.2.91).
+ *  \param[out] ti Talker Identity structure to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_talker_identity(struct gsm0808_talker_identity *ti, const uint8_t *elem, uint8_t len)
+{
+	OSMO_ASSERT(ti);
+	OSMO_ASSERT(elem);
+
+	if (len < 2)
+		return -EINVAL;
+	unsigned int bytes;
+
+	bytes = len - 1;
+	if (bytes > TALKER_IDENTITY_MAXLEN)
+		return -EINVAL;
+	ti->id_bits = (bytes << 3) - (elem[0] & 0x7);
+	memcpy(ti->talker_id, elem + 1, bytes);
+
+	return len;
+}
+
+/*! Encode Assignment Requirements IE (3GPP TS 48.008 3.2.2.52).
+ *  \param[out] msg Message Buffer to which IE is to be appended
+ *  \param[in] ar Assignment Requirement to be encoded
+ *  \returns number of bytes appended to \a msg */
+uint8_t gsm0808_enc_assign_req(struct msgb *msg, const enum gsm0808_assignment_requirement ar)
+{
+	uint8_t *ptr;
+
+	OSMO_ASSERT(msg);
+
+	msgb_put_u8(msg, GSM0808_IE_ASSIGNMENT_REQUIREMENT);
+
+	ptr = msgb_put(msg, 1);
+	ptr[0] = ar;
+
+	return 2;
+}
+
+/*! Decode Assignment Requirements IE (3GPP TS 48.008 3.2.2.52).
+ *  \param[out] ar Assignment Requirements enum to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_assign_req(enum gsm0808_assignment_requirement *ar, const uint8_t *elem, uint8_t len)
+{
+	OSMO_ASSERT(ar);
+	OSMO_ASSERT(elem);
+
+	if (len != 1)
+		return -EINVAL;
+
+	*ar = elem[0];
+
+	return 1;
+}
+
+/*! Encode Cell Identifier List Segment IE (3GPP TS 48.008 3.2.2.27a).
+ *  \param[out] msg Message Buffer to which IE is to be appended
+ *  \param[in] ie_type Type of IE to use (5 different lists are specified.)
+ *  \param[in] ci Cell Identifier List Segment to be encoded
+ *  \returns number of bytes appended to \a msg */
+uint8_t gsm0808_enc_cell_id_list_segment(struct msgb *msg, uint8_t ie_type,
+					 const struct gsm0808_cell_id_list_segment *ci)
+{
+	uint8_t *old_tail;
+	uint8_t *tlv_len;
+	int rc;
+
+	OSMO_ASSERT(msg);
+	OSMO_ASSERT(ci);
+
+	msgb_put_u8(msg, ie_type);
+	tlv_len = msgb_put(msg, 1);
+	old_tail = msg->tail;
+
+	msgb_put_u8(msg, (ci->seq_last << 4) | ci->seq_number);
+
+	rc = gsm0808_enc_cell_id_list2(msg, &ci->cil);
+	if (rc <= 0)
+		return rc;
+
+	*tlv_len = (uint8_t) (msg->tail - old_tail);
+	return *tlv_len + 2;
+}
+
+/*! Decode Cell Identifier List Segment IE (3GPP TS 48.008 3.2.2.27a).
+ *  \param[out] ci Cell Identifier List Segment structure to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_cell_id_list_segment(struct gsm0808_cell_id_list_segment *ci, const uint8_t *elem, uint8_t len)
+{
+	int rc;
+
+	OSMO_ASSERT(ci);
+	OSMO_ASSERT(elem);
+
+	if (len < 1)
+		return -EINVAL;
+
+	ci->seq_last = elem[0] >> 4;
+	ci->seq_number = elem[0] & 0x0f;
+
+	rc = gsm0808_dec_cell_id_list2(&ci->cil, elem + 1, len - 1);
+	if (rc < 0)
+		return rc;
+
+	return len;
+}
+
+/*! Decode Call Identifier IE (3GPP TS 48.008 3.2.2.105).
+ *  \param[out] ci Call Identifier structure to store data
+ *  \param[in] elem IE value to be decoded.
+ *  \param[in] len Length of \a elem in bytes.
+ *  \returns number of bytes parsed; negative on error */
+int gsm0808_dec_call_id(uint32_t *ci, const uint8_t *elem, uint8_t len)
+{
+	OSMO_ASSERT(ci);
+	OSMO_ASSERT(elem);
+
+	if (len != 4)
+		return -EINVAL;
+
+	/* The Call Identifier is stored as little endian. */
+	*ci = osmo_load32le(elem);
+
+	return 4;
+}
+
 /*! @} */
