@@ -126,6 +126,7 @@ const struct tlv_definition rsl_att_tlvdef = {
 		[RSL_IE_OSMO_TRAINING_SEQUENCE]	= { TLV_TYPE_TLV },
 		[RSL_IE_OSMO_TEMP_OVP_ACCH_CAP]	= { TLV_TYPE_TLV },
 		[RSL_IE_OSMO_OSMUX_CID]		= { TLV_TYPE_TLV },
+		[RSL_IE_OSMO_RTP_EXTENSIONS]	= { TLV_TYPE_TLV },
 		[RSL_IE_IPAC_SRTP_CONFIG]	= { TLV_TYPE_TLV },
 		[RSL_IE_IPAC_PROXY_UDP]		= { TLV_TYPE_FIXED, 2 },
 		[RSL_IE_IPAC_BSCMPL_TOUT]	= { TLV_TYPE_TV },
@@ -553,7 +554,7 @@ void rsl_rll_push_l3(struct msgb *msg, uint8_t msg_type, uint8_t chan_nr,
 	/* construct a RSLms RLL message (DATA INDICATION, UNIT DATA
 	 * INDICATION) and send it off via RSLms */
 
-	/* Push the L3 IE tag and lengh */
+	/* Push the L3 IE tag and length */
 	msgb_tv16_push(msg, RSL_IE_L3_INFO, l3_len);
 
 	/* Then push the RSL header */
